import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:persistent_bottom_nav_bar_v2/persistent-tab-view.dart';

Widget defaultScreen(int id) => Container(child: Text("Screen$id"));

Widget screenWithTextField(int id) =>
    Column(children: [defaultScreen(id), TextFormField()]);

void main() {
  Widget wrapTabView(WidgetBuilder builder) {
    return MaterialApp(
      home: Builder(
        builder: (context) => builder(context),
      ),
    );
  }

  group('PersistentTabView', () {
    List<PersistentBottomNavBarItem> items = [
      PersistentBottomNavBarItem(title: "Item1", icon: Icon(Icons.add)),
      PersistentBottomNavBarItem(title: "Item2", icon: Icon(Icons.add)),
      PersistentBottomNavBarItem(title: "Item3", icon: Icon(Icons.add)),
    ];

    testWidgets('builds a PersistentBottomNavBar', (WidgetTester tester) async {
      await tester.pumpWidget(
        wrapTabView(
          (context) => PersistentTabView(
            context,
            screens: [1, 2, 3].map((id) => defaultScreen(id)).toList(),
            items: items,
            navBarStyle: NavBarStyle.style3,
          ),
        ),
      );

      expect(find.byType(PersistentBottomNavBar).hitTestable(), findsOneWidget);
    });

    testWidgets('shows all tabs and the correct screen for it',
        (WidgetTester tester) async {
      await tester.pumpWidget(
        wrapTabView(
          (context) => PersistentTabView(
            context,
            screens: [1, 2, 3].map((id) => defaultScreen(id)).toList(),
            items: items,
            navBarStyle: NavBarStyle.style3,
          ),
        ),
      );

      expect(find.text('Screen1'), findsOneWidget);
      expect(find.text('Screen2'), findsNothing);
      expect(find.text('Screen3'), findsNothing);
      await tester.tap(find.text("Item2"));
      await tester.pumpAndSettle();
      expect(find.text('Screen1'), findsNothing);
      expect(find.text('Screen2'), findsOneWidget);
      expect(find.text('Screen3'), findsNothing);
      await tester.tap(find.text("Item3"));
      await tester.pumpAndSettle();
      expect(find.text('Screen1'), findsNothing);
      expect(find.text('Screen2'), findsNothing);
      expect(find.text('Screen3'), findsOneWidget);
      await tester.tap(find.text("Item1"));
      await tester.pumpAndSettle();
      expect(find.text('Screen1'), findsOneWidget);
      expect(find.text('Screen2'), findsNothing);
      expect(find.text('Screen3'), findsNothing);
    });

    testWidgets('switches screens when tapping on items',
        (WidgetTester tester) async {
      await tester.pumpWidget(
        wrapTabView(
          (context) => PersistentTabView(
            context,
            screens: [1, 2, 3].map((id) => defaultScreen(id)).toList(),
            items: items,
            navBarStyle: NavBarStyle.style3,
          ),
        ),
      );

      expect(find.text('Screen1'), findsOneWidget);
      expect(find.text('Screen2'), findsNothing);
      await tester.tap(find.text("Item2"));
      await tester.pumpAndSettle();
      expect(find.text('Screen1'), findsNothing);
      expect(find.text('Screen2'), findsOneWidget);
    });

    testWidgets('hides the navbar when hideNavBar is true',
        (WidgetTester tester) async {
      await tester.pumpWidget(
        wrapTabView(
          (context) => PersistentTabView(
            context,
            screens: [1, 2, 3].map((id) => defaultScreen(id)).toList(),
            items: items,
            navBarStyle: NavBarStyle.style3,
            hideNavigationBar: true,
          ),
        ),
      );

      expect(find.byType(PersistentBottomNavBar).hitTestable(), findsNothing);
    });

    testWidgets(
        'hides the navbar when hideNavigationBarWhenKeyboardShows is true and keyboard is up',
        (WidgetTester tester) async {
      await tester.pumpWidget(
        wrapTabView(
          (context) => PersistentTabView(
            context,
            screens: [1, 2, 3].map((id) => screenWithTextField(id)).toList(),
            items: items,
            navBarStyle: NavBarStyle.style3,
            hideNavigationBarWhenKeyboardShows: true,
          ),
        ),
      );

      expect(find.byType(PersistentBottomNavBar).hitTestable(), findsOneWidget);
      await tester.tap(find.byType(TextFormField));
      await tester.showKeyboard(find.byType(TextFormField));
      await tester.pumpAndSettle();
      expect(find.byType(PersistentBottomNavBar).hitTestable(), findsNothing);
    });
  });
}
